@@ -1,5 +1,4 @@
 require 'puppet/application'
-require 'puppet/configurer'
 require 'puppet/util/network_device'
 
 class Puppet::Application::Device < Puppet::Application
@@ -54,10 +53,6 @@ class Puppet::Application::Device < Puppet::Application
     options[:detailed_exitcodes] = true
   end
 
-  option("--libdir LIBDIR") do |arg|
-    options[:libdir] = arg
-  end
-
   option("--apply MANIFEST") do |arg|
     options[:apply] = arg.to_s
   end
@@ -98,11 +93,10 @@ a scheduled task, or a similar tool.
 
 USAGE
 -----
-  puppet device [-h|--help] [-v|--verbose] [-d|--debug]
-                [-l|--logdest syslog|<file>|console] [--detailed-exitcodes]
-                [--deviceconfig <file>] [-w|--waitforcert <seconds>]
-                [--libdir <directory>]
-                [-a|--apply <file>] [-f|--facts] [-r|--resource <type> [name]]
+  puppet device [-d|--debug] [--detailed-exitcodes] [--deviceconfig <file>]
+                [-h|--help] [-l|--logdest syslog|<file>|console]
+                [-v|--verbose] [-w|--waitforcert <seconds>] [-f|--facts]
+                [-a|--apply <file>] [-r|--resource <type> [name]]
                 [-t|--target <device>] [--user=<user>] [-V|--version]
 
 
@@ -141,25 +135,9 @@ Note that any setting that's valid in the configuration file is also a valid
 long argument. For example, 'server' is a valid configuration parameter, so
 you can specify '--server <servername>' as an argument.
 
-* --help, -h:
-  Print this help message
-
-* --verbose, -v:
-  Turn on verbose reporting.
-
-* --debug, -d:
+* --debug:
   Enable full debugging.
 
-* --logdest, -l:
-  Where to send log messages. Choose between 'syslog' (the POSIX syslog
-  service), 'console', or the path to a log file. If debugging or verbosity is
-  enabled, this defaults to 'console'. Otherwise, it defaults to 'syslog'.
-
-  A path ending with '.json' will receive structured output in JSON format. The
-  log file will not have an ending ']' automatically written to it due to the
-  appending nature of logging. It must be appended manually to make the content
-  valid JSON.
-
 * --detailed-exitcodes:
   Provide transaction information via exit codes. If this is enabled, an exit
   code of '1' means at least one device had a compile failure, an exit code of
@@ -171,16 +149,18 @@ you can specify '--server <servername>' as an argument.
   Path to the device config file for puppet device.
   Default: $confdir/device.conf
 
-* --waitforcert, -w:
-  This option only matters for targets that do not yet have certificates
-  and it is enabled by default, with a value of 120 (seconds).  This causes
-  +puppet device+ to poll the server every 2 minutes and ask it to sign a
-  certificate request.  This is useful for the initial setup of a target.
-  You can turn off waiting for certificates by specifying a time of 0.
+* --help:
+  Print this help message
 
-* --libdir:
-  Override the per-device libdir with a local directory. Specifying a libdir also
-  disables pluginsync. This is useful for testing.
+* --logdest:
+  Where to send log messages. Choose between 'syslog' (the POSIX syslog
+  service), 'console', or the path to a log file. If debugging or verbosity is
+  enabled, this defaults to 'console'. Otherwise, it defaults to 'syslog'.
+
+  A path ending with '.json' will receive structured output in JSON format. The
+  log file will not have an ending ']' automatically written to it due to the
+  appending nature of logging. It must be appended manually to make the content
+  valid JSON.
 
 * --apply:
   Apply a manifest against a remote target. Target must be specified.
@@ -203,6 +183,16 @@ you can specify '--server <servername>' as an argument.
 * --user:
   The user to run as.
 
+* --verbose:
+  Turn on verbose reporting.
+
+* --waitforcert:
+  This option only matters for daemons that do not yet have certificates
+  and it is enabled by default, with a value of 120 (seconds).  This causes
+  +puppet agent+ to connect to the server every 2 minutes and ask it to sign a
+  certificate request.  This is useful for the initial setup of a puppet
+  client.  You can turn off waiting for certificates by specifying a time of 0.
+
 
 EXAMPLE
 -------
@@ -215,7 +205,7 @@ Brice Figureau
 
 COPYRIGHT
 ---------
-Copyright (c) 2011-2018 Puppet Inc., LLC
+Copyright (c) 2011 Puppet Inc., LLC
 Licensed under the Apache 2.0 License
       HELP
   end
@@ -232,12 +222,11 @@ Licensed under the Apache 2.0 License
       raise _("missing argument: --target is required when using --apply") if options[:target].nil?
       raise _("%{file} does not exist, cannot apply") % { file: options[:apply] } unless File.file?(options[:apply])
     end
-    libdir = Puppet[:libdir]
     vardir = Puppet[:vardir]
     confdir = Puppet[:confdir]
     certname = Puppet[:certname]
 
-    env = Puppet::Node::Environment.remote(Puppet[:environment])
+    env = Puppet.lookup(:environments).get(Puppet[:environment])
     returns = Puppet.override(:current_environment => env, :loaders => Puppet::Pops::Loaders.new(env)) do
       # find device list
       require 'puppet/util/network_device/config'
@@ -262,33 +251,10 @@ Licensed under the Apache 2.0 License
 
           # override local $vardir and $certname
           Puppet[:confdir] = ::File.join(Puppet[:devicedir], device.name)
-          Puppet[:libdir] = options[:libdir] || ::File.join(Puppet[:devicedir], device.name, 'lib')
           Puppet[:vardir] = ::File.join(Puppet[:devicedir], device.name)
           Puppet[:certname] = device.name
 
-          unless options[:resource] || options[:facts] || options[:apply]
-            # this will reload and recompute default settings and create the devices sub vardir
-            Puppet.settings.use :main, :agent, :ssl
-
-            # Since it's too complicated to fix properly in the default settings, we workaround for PUP-9642 here.
-            # See https://github.com/puppetlabs/puppet/pull/7483#issuecomment-483455997 for details.
-            # This has to happen after `settings.use` above, so the directory is created and before `setup_host` below, where the SSL
-            # routines would fail with access errors
-            if Puppet.features.root? && !Puppet::Util::Platform.windows?
-              user = Puppet::Type.type(:user).new(name: Puppet[:user]).exists? ? Puppet[:user] : nil
-              group = Puppet::Type.type(:group).new(name: Puppet[:group]).exists? ? Puppet[:group] : nil
-              Puppet.debug("Fixing perms for #{user}:#{group} on #{Puppet[:confdir]}")
-              FileUtils.chown(user, group, Puppet[:confdir]) if user || group
-            end
-
-            # ask for a ssl cert if needed, and setup the ssl system for this device.
-            setup_host
-
-            # only pluginsync if we do not have a libdir specified on the command line
-            Puppet::Configurer::PluginHandler.new.download_plugins(env) unless options[:libdir]
-          end
-
-          # this inits the device singleton, so that the facts terminus
+          # this init the device singleton, so that the facts terminus
           # and the various network_device provider can use it
           Puppet::Util::NetworkDevice.init(device)
 
@@ -297,10 +263,10 @@ Licensed under the Apache 2.0 License
             Puppet.info _("retrieving resource: %{resource} from %{target} at %{scheme}%{url_host}%{port}%{url_path}") % { resource: type, target: device.name, scheme: scheme, url_host: device_url.host, port: port, url_path: device_url.path }
             resources = find_resources(type, name)
             if options[:to_yaml]
-              data = resources.map do |resource|
-                resource.prune_parameters(:parameters_to_include => @extra_params).to_hiera_hash
-              end.inject(:merge!)
-              text = YAML.dump(type.downcase => data)
+              text = resources.map do |resource|
+                resource.prune_parameters(:parameters_to_include => @extra_params).to_hierayaml.force_encoding(Encoding.default_external)
+              end.join("\n")
+              text.prepend("#{type.downcase}:\n")
             else
               text = resources.map do |resource|
                 resource.prune_parameters(:parameters_to_include => @extra_params).to_manifest.force_encoding(Encoding.default_external)
@@ -335,15 +301,21 @@ Licensed under the Apache 2.0 License
             end
           else
             Puppet.info _("starting applying configuration to %{target} at %{scheme}%{url_host}%{port}%{url_path}") % { target: device.name, scheme: scheme, url_host: device_url.host, port: port, url_path: device_url.path }
+            # this will reload and recompute default settings and create the devices sub vardir
+            Puppet.settings.use :main, :agent, :ssl
+            # ask for a ssl cert if needed, but at least
+            # setup the ssl system for this device.
+            setup_host
+
+            require 'puppet/configurer'
             configurer = Puppet::Configurer.new
-            configurer.run(:network_device => true, :pluginsync => Puppet::Configurer.should_pluginsync? && !options[:libdir])
+            configurer.run(:network_device => true, :pluginsync => Puppet::Configurer.should_pluginsync?)
           end
         rescue => detail
           Puppet.log_exception(detail)
           # If we rescued an error, then we return 1 as the exit code
           1
         ensure
-          Puppet[:libdir] = libdir
           Puppet[:vardir] = vardir
           Puppet[:confdir] = confdir
           Puppet[:certname] = certname
@@ -404,11 +376,6 @@ Licensed under the Apache 2.0 License
 
       Puppet.settings.use :main, :agent, :device, :ssl
 
-      # We need to specify a ca location for all of the SSL-related
-      # indirected classes to work; in fingerprint mode we just need
-      # access to the local files and we don't need a ca.
-      Puppet::SSL::Host.ca_location = :remote
-
       Puppet::Transaction::Report.indirection.terminus_class = :rest
 
       if Puppet[:catalog_cache_terminus]
