@@ -32,6 +32,32 @@ module Puppet
       %w[md5 md5lite sha256 sha256lite sha384 sha512 sha224 sha1 sha1lite mtime ctime]
   end
 
+  def self.default_basemodulepath
+    if Puppet::Util::Platform.windows?
+      path = ['$codedir/modules']
+      installdir = Facter.value(:env_windows_installdir)
+      if installdir
+        path << "#{installdir}/puppet/modules"
+      end
+      path.join(File::PATH_SEPARATOR)
+    else
+      '$codedir/modules:/opt/puppetlabs/puppet/modules'
+    end
+  end
+
+  def self.default_vendormoduledir
+    if Puppet::Util::Platform.windows?
+      installdir = Facter.value(:env_windows_installdir)
+      if installdir
+        "#{installdir}\\puppet\\vendor_modules"
+      else
+        nil
+      end
+    else
+      '/opt/puppetlabs/puppet/vendor_modules'
+    end
+  end
+
   ############################################################################################
   # NOTE: For information about the available values for the ":type" property of settings,
   #   see the docs for Settings.define_settings
@@ -39,11 +65,7 @@ module Puppet
 
   AS_DURATION = %q{This setting can be a time interval in seconds (30 or 30s), minutes (30m), hours (6h), days (2d), or years (5y).}
 
-  # @api public
-  # @param args [Puppet::Settings] the settings object to define default settings for
-  # @return void
-  def self.initialize_default_settings!(settings)
-  settings.define_settings(:main,
+  define_settings(:main,
     :confdir  => {
         :default  => nil,
         :type     => :directory,
@@ -80,7 +102,7 @@ module Puppet
     }
   )
 
-  settings.define_settings(:main,
+  define_settings(:main,
     :logdir => {
         :default  => nil,
         :type     => :directory,
@@ -171,7 +193,7 @@ module Puppet
     }
   )
 
-  settings.define_settings(:main,
+  define_settings(:main,
     :priority => {
       :default => nil,
       :type    => :priority,
@@ -184,19 +206,12 @@ module Puppet
     :trace => {
         :default  => false,
         :type     => :boolean,
-        :desc     => "Whether to print stack traces on some errors. Will print
-          internal Ruby stack trace interleaved with Puppet function frames.",
+        :desc     => "Whether to print stack traces on some errors",
         :hook     => proc do |value|
           # Enable or disable Facter's trace option too
           Facter.trace(value) if Facter.respond_to? :trace
         end
     },
-    :puppet_trace => {
-        :default  => false,
-        :type     => :boolean,
-        :desc     => "Whether to print the Puppet stack trace on some errors.
-          This is a noop if `trace` is also set.",
-    },
     :profile => {
         :default  => false,
         :type     => :boolean,
@@ -325,13 +340,7 @@ module Puppet
           for those files that Puppet will load on demand, and is only
           guaranteed to work for those cases.  In fact, the autoload
           mechanism is responsible for making sure this directory
-          is in Ruby's search path\n",
-      :call_hook => :on_initialize_and_write,
-      :hook             => proc do |value|
-        $LOAD_PATH.delete(@oldlibdir) if defined?(@oldlibdir) && $LOAD_PATH.include?(@oldlibdir)
-        @oldlibdir = value
-        $LOAD_PATH << value
-      end
+          is in Ruby's search path\n"
     },
     :environment => {
         :default  => "production",
@@ -394,7 +403,7 @@ module Puppet
           files. The command to use can be chosen with the `diff` setting.",
     },
     :diff => {
-      :default => (Puppet.features.microsoft_windows? ? "" : "diff"),
+      :default => (Puppet::Util::Platform.windows? ? "" : "diff"),
       :desc    => "Which diff command to use when printing differences between files. This setting
           has no default value on Windows, as standard `diff` is not available, but Puppet can use many
           third-party diff tools.",
@@ -411,13 +420,13 @@ module Puppet
     },
     :daemonize => {
         :type     => :boolean,
-        :default  => (Puppet.features.microsoft_windows? ? false : true),
+        :default  => (Puppet::Util::Platform.windows? ? false : true),
         :desc     => "Whether to send the process into the background.  This defaults
           to true on POSIX systems, and to false on Windows (where Puppet
           currently cannot daemonize).",
         :short    => "D",
         :hook     => proc do |value|
-          if value and Puppet.features.microsoft_windows?
+          if value and Puppet::Util::Platform.windows?
             raise "Cannot daemonize on Windows"
           end
       end
@@ -465,7 +474,7 @@ module Puppet
       :type       => :terminus,
       :default    => nil,
       :desc       => "How to store cached nodes.
-      Valid values are (none), 'json', 'msgpack', 'yaml' or write only yaml ('write_only_yaml').",
+      Valid values are (none), 'json', 'msgpack', or 'yaml'.",
     },
     :data_binding_terminus => {
       :type    => :terminus,
@@ -492,12 +501,12 @@ module Puppet
     :hiera_config => {
       :default => lambda do
         config = nil
-        codedir = settings[:codedir]
+        codedir = Puppet.settings[:codedir]
         if codedir.is_a?(String)
           config = File.expand_path(File.join(codedir, 'hiera.yaml'))
           config = nil unless Puppet::FileSystem.exist?(config)
         end
-        config = File.expand_path(File.join(settings[:confdir], 'hiera.yaml')) if config.nil?
+        config = File.expand_path(File.join(Puppet.settings[:confdir], 'hiera.yaml')) if config.nil?
         config
       end,
       :desc    => "The hiera configuration file. Puppet only reads this file on startup, so you must restart the puppet master every time you edit it.",
@@ -535,10 +544,9 @@ module Puppet
     },
     :http_proxy_host => {
       :default    => "none",
-      :desc       => "The HTTP proxy host to use for outgoing connections. The proxy will be bypassed if
-      the server's hostname matches the NO_PROXY environment variable or `no_proxy` setting. Note: You
+      :desc       => "The HTTP proxy host to use for outgoing connections.  Note: You
       may need to use a FQDN for the server hostname when using a proxy. Environment variable
-      http_proxy or HTTP_PROXY will override this value. ",
+      http_proxy or HTTP_PROXY will override this value",
     },
     :http_proxy_port => {
       :default    => 3128,
@@ -551,7 +559,7 @@ module Puppet
     :http_proxy_password =>{
       :default    => "none",
       :hook       => proc do |value|
-        if settings[:http_proxy_password] =~ /[@!# \/]/
+        if Puppet.settings[:http_proxy_password] =~ /[@!# \/]/
           raise "Passwords set in the http_proxy_password setting must be valid as part of a URL, and any reserved characters must be URL-encoded. We received: #{value}"
         end
       end,
@@ -562,10 +570,6 @@ module Puppet
         contains any characters with special meanings in URLs (as specified by RFC 3986
         section 2.2), they must be URL-encoded. (For example, `#` would become `%23`.)",
     },
-    :no_proxy => {
-      :default    => "localhost, 127.0.0.1",
-      :desc       => "List of host or domain names that should not go through `http_proxy_host`. Environment variable no_proxy or NO_PROXY will override this value. Names can be specified as an FQDN `host.example.com`, wildcard `*.example.com`, dotted domain `.example.com`, or suffix `example.com`.",
-    },
     :http_keepalive_timeout => {
       :default    => "4s",
       :type       => :duration,
@@ -585,9 +589,10 @@ module Puppet
       #{AS_DURATION}",
     },
     :http_read_timeout => {
+      :default => "10m",
       :type    => :duration,
-      :desc    => "The time to wait for one block to be read from an HTTP connection. If nothing is
-      read after the elapsed interval then the connection will be closed. The default value is unlimited.
+      :desc    => "The time to wait for data to be read from an HTTP connection. If nothing is
+      read after the elapsed interval then the connection will be closed. The default value is 10 minutes.
       #{AS_DURATION}",
     },
     :http_user_agent => {
@@ -664,13 +669,6 @@ module Puppet
           essentially means that you can't have any code outside of a node,
           class, or definition other than in the site manifest.",
     },
-    :trusted_server_facts => {
-      :default => true,
-      :type    => :boolean,
-      :deprecated => :completely,
-      :desc    => "The 'trusted_server_facts' setting is deprecated and has no effect as the
-        feature this enabled is now always on. The setting will be removed in a future version of puppet.",
-    },
     :preview_outputdir => {
       :default => '$vardir/preview',
       :type     => :directory,
@@ -681,29 +679,15 @@ module Puppet
     }
   )
 
-  settings.define_settings(:main,
-      # Whether the application management feature is on or off - now deprecated and always on.
-      :app_management => {
-          :default  => false,
-          :type     => :boolean,
-          :desc       => "This setting has no effect and will be removed in a future Puppet version.",
-          :deprecated => :completely,
-      }
-  )
-
-  settings.define_settings(:module_tool,
+  Puppet.define_settings(:module_tool,
     :module_repository  => {
       :default  => 'https://forgeapi.puppet.com',
       :desc     => "The module repository",
     },
     :module_working_dir => {
-        :default  => (Puppet.features.microsoft_windows? ? Dir.tmpdir() : '$vardir/puppet-module'),
+        :default  => (Puppet::Util::Platform.windows? ? Dir.tmpdir() : '$vardir/puppet-module'),
         :desc     => "The directory into which module tool data is stored",
     },
-    :module_skeleton_dir => {
-        :default  => '$module_working_dir/skeleton',
-        :desc     => "The directory which the skeleton for module tool generate is stored.",
-    },
     :forge_authorization => {
         :default  => nil,
         :desc     => "The authorization key to connect to the Puppet Forge. Leave blank for unauthorized or license based connections",
@@ -714,7 +698,7 @@ module Puppet
     }
   )
 
-    settings.define_settings(
+    Puppet.define_settings(
     :main,
 
     # We have to downcase the fqdn, because the current ssl stuff (as opposed to in master) doesn't have good facilities for
@@ -750,9 +734,8 @@ module Puppet
       :desc    => <<EOT,
 A comma-separated list of alternate DNS names for Puppet Server. These are extra
 hostnames (in addition to its `certname`) that the server is allowed to use when
-serving agents. Puppet checks this setting when automatically requesting a
-certificate for Puppet agent or Puppet Server, and when manually generating a
-certificate with `puppet cert generate`. These can be either IP or DNS, and the type
+serving agents. Puppet checks this setting when automatically creating a
+certificate for Puppet agent or Puppet Server. These can be either IP or DNS, and the type
 should be specified and followed with a colon. Untyped inputs will default to DNS.
 
 In order to handle agent requests at a given hostname (like
@@ -765,23 +748,12 @@ names.
 
 **Note:** The list of alternate names is locked in when the server's
 certificate is signed. If you need to change the list later, you can't just
-change this setting; you also need to:
-
-* On the server: Stop Puppet Server.
-* On the CA server: Revoke and clean the server's old certificate. (`puppet cert clean <NAME>`)
-  (Note `puppet cert clean` is deprecated and will be replaced with `puppetserver ca clean`
-  in Puppet 6.)
-* On the server: Delete the old certificate (and any old certificate signing requests)
-  from the [ssldir](https://puppet.com/docs/puppet/latest/dirs_ssldir.html).
-* On the server: Run `puppet agent -t --ca_server <CA HOSTNAME>` to request a new certificate
-* On the CA server: Sign the certificate request, explicitly allowing alternate names
-  (`puppet cert sign --allow-dns-alt-names <NAME>`). (Note `puppet cert sign` is deprecated
-  and will be replaced with `puppetserver ca sign` in Puppet 6.)
-* On the server: Run `puppet agent -t --ca_server <CA HOSTNAME>` to retrieve the cert.
-* On the server: Start Puppet Server again.
+change this setting; you also need to regenerate the certificate. For more
+information on that process, see the [cert regen docs]
+(https://puppet.com/docs/puppet/latest/ssl_regenerate_certificates.html).
 
 To see all the alternate names your servers are using, log into your CA server
-and run `puppet cert list -a`, then check the output for `(alt names: ...)`.
+and run `puppetserver ca list --all`, then check the output for `(alt names: ...)`.
 Most agent nodes should NOT have alternate names; the only certs that should
 have them are Puppet Server nodes that you want other agents to trust.
 EOT
@@ -793,7 +765,7 @@ EOT
 An optional file containing custom attributes to add to certificate signing
 requests (CSRs). You should ensure that this file does not exist on your CA
 puppet master; if it does, unwanted certificate extensions may leak into
-certificates created with the `puppet cert generate` command.
+certificates created with the `puppetserver ca generate` command.
 
 If present, this file must be a YAML hash containing a `custom_attributes` key
 and/or an `extension_requests` key. The value of each key must be a hash, where
@@ -807,17 +779,13 @@ This is useful for embedding a pre-shared key for autosigning policy executables
 ("challenge password") OID.
 
 Extension requests will be permanently embedded in the final certificate.
-Extension OIDs must be in the "ppRegCertExt" (`1.3.6.1.4.1.34380.1.1`),
-"ppPrivCertExt" (`1.3.6.1.4.1.34380.1.2`), or
-"ppAuthCertExt" (`1.3.6.1.4.1.34380.1.3`) OID arcs. The ppRegCertExt arc is
+Extension OIDs must be in the "ppRegCertExt" (`1.3.6.1.4.1.34380.1.1`) or
+"ppPrivCertExt" (`1.3.6.1.4.1.34380.1.2`) OID arcs. The ppRegCertExt arc is
 reserved for four of the most common pieces of data to embed: `pp_uuid` (`.1`),
 `pp_instance_id` (`.2`), `pp_image_name` (`.3`), and `pp_preshared_key` (`.4`)
 --- in the YAML file, these can be referred to by their short descriptive names
 instead of their full OID. The ppPrivCertExt arc is unregulated, and can be used
-for site-specific extensions. The ppAuthCert arc is reserved for two pieces of
-data to embed: `pp_authorization` (`.1`) and `pp_auth_role` (`.13`). As with
-ppRegCertExt, in the YAML file, these can be referred to by their short
-descriptive name instead of their full OID.
+for site-specific extensions.
 EOT
     },
     :certdir => {
@@ -949,25 +917,26 @@ EOT
     :certificate_revocation => {
         :default  => 'chain',
         :type     => :certificate_revocation,
-        :desc     => <<EOT
-Whether certificate revocation checking should be enabled, and what level of checking should be performed.
+        :desc     => <<-'EOT'
+          Whether certificate revocation checking should be enabled, and what level of
+          checking should be performed.
 
-When certificate_revocation is set to 'true' or 'chain', Puppet will download the CA CRL and will perform revocation
-checking against each certificate in the chain.
+          When certificate revocation is enabled Puppet expects the contents of its CRL
+          to be one or more PEM encoded CRLs concatenated together. When using a cert
+          bundle CRLs for all CAs in the chain of trust must be included in the crl file
+          with the first CRL listed being for the root of the chain, the last being for
+          the leaf CA.
 
-Puppet is unable to load multiple CRLs, so if certificate_revocation is set to 'chain' and Puppet attempts to verify
-a certificate signed by a root CA the behavior is equivalent to the 'leaf' setting, and if Puppet attempts to verify
-a certificate signed by an intermediate CA then verification will fail as Puppet will be unable to load the multiple
-CRLs required for full chain checking. As such the 'chain' setting is limited in functionality and is meant as a stand
-in pending the implementation of full chain checking.
+          When certificate_revocation is set to 'true' or 'chain', Puppet will ensure
+          that each CA in the chain of trust has not been revoked by its issuing CA.
 
-When certificate_revocation is set to 'leaf', Puppet will download the CA CRL and will verify the leaf certificate
-against that CRL. CRLs will not be fetched or checked for the rest of the certificates in the chain. If you are using
-an intermediate CA certificate and want to enable certificate revocation checking, this setting must be set to 'leaf'.
+          When certificate_revocation is set to 'leaf', Puppet will verify certs against
+          the issuing CA's revocation list but not verify the revocation status of the
+          issuing CA or any CA above it within the chain of trust.
 
-When certificate_revocation is set to 'false', Puppet will disable all certificate revocation checking and will not
-attempt to download the CRL.
-EOT
+          When certificate_revocation is set to 'false', Puppet will disable all
+          certificate revocation checking and will not attempt to download the CRL.
+        EOT
     },
     :digest_algorithm => {
         :default  => lambda { default_digest_algorithm },
@@ -1006,96 +975,50 @@ EOT
     }
   )
 
-    settings.define_settings(
+    define_settings(
     :ca,
     :ca_name => {
-      :default    => "Puppet CA: $certname",
-      :desc       => "The name to use the Certificate Authority certificate.",
+      :default => "Puppet CA: $certname",
+      :desc    => "The name to use the Certificate Authority certificate.",
     },
     :cadir => {
       :default => "$ssldir/ca",
       :type => :directory,
-      :owner => "service",
-      :group => "service",
-      :mode => "0755",
       :desc => "The root directory for the certificate authority.",
     },
     :cacert => {
       :default => "$cadir/ca_crt.pem",
       :type => :file,
-      :owner => "service",
-      :group => "service",
-      :mode => "0644",
       :desc => "The CA certificate.",
     },
     :cakey => {
       :default => "$cadir/ca_key.pem",
       :type => :file,
-      :owner => "service",
-      :group => "service",
-      :mode => "0640",
       :desc => "The CA private key.",
     },
     :capub => {
       :default => "$cadir/ca_pub.pem",
       :type => :file,
-      :owner => "service",
-      :group => "service",
-      :mode => "0644",
       :desc => "The CA public key.",
     },
     :cacrl => {
       :default => "$cadir/ca_crl.pem",
       :type => :file,
-      :owner => "service",
-      :group => "service",
-      :mode => "0644",
-      :desc => "The certificate revocation list (CRL) for the CA. Will be used if present but otherwise ignored.",
-    },
-    :caprivatedir => {
-      :default => "$cadir/private",
-      :type => :directory,
-      :owner => "service",
-      :group => "service",
-      :mode => "0750",
-      :desc => "Where the CA stores private certificate information. This setting is deprecated and will be removed in Puppet 6.",
-      :hook => proc do |value|
-        Puppet.deprecation_warning(_("The 'caprivatedir' setting is deprecated and will be removed in Puppet 6."))
-      end,
+      :desc => "The certificate revocation list (CRL) for the CA.",
     },
     :csrdir => {
       :default => "$cadir/requests",
       :type => :directory,
-      :owner => "service",
-      :group => "service",
-      :mode  => "0755",
       :desc => "Where the CA stores certificate requests.",
     },
     :signeddir => {
       :default => "$cadir/signed",
       :type => :directory,
-      :owner => "service",
-      :group => "service",
-      :mode => "0755",
       :desc => "Where the CA stores signed certificates.",
     },
-    :capass => {
-      :default => "$caprivatedir/ca.pass",
-      :type => :file,
-      :owner => "service",
-      :group => "service",
-      :mode => "0640",
-      :desc => "Where the CA stores the password for the private key. This setting is deprecated and will be removed in Puppet 6.",
-      :hook => proc do |value|
-        Puppet.deprecation_warning(_("The 'capass' setting is deprecated and will be removed in Puppet 6."))
-      end,
-    },
     :serial => {
       :default => "$cadir/serial",
       :type => :file,
-      :owner => "service",
-      :group => "service",
-      :mode => "0644",
       :desc => "Where the serial number for certificates is stored.",
     },
     :autosign => {
@@ -1120,7 +1043,7 @@ EOT
         and non-zero if the cert should not be autosigned.
 
         If a certificate request is not autosigned, it will persist for review. An admin
-        user can use the `puppet cert sign` command to manually sign it, or can delete
+        user can use the `puppetserver ca sign` command to manually sign it, or can delete
         the request.
 
         For info on autosign configuration files, see
@@ -1144,9 +1067,6 @@ EOT
     :cert_inventory => {
       :default => "$cadir/inventory.txt",
       :type => :file,
-      :mode => "0644",
-      :owner => "service",
-      :group => "service",
       :desc => "The inventory file. This is a text file to which the CA writes a
         complete listing of all certificates.",
     }
@@ -1154,7 +1074,7 @@ EOT
 
   # Define the config default.
 
-    settings.define_settings(:application,
+    define_settings(:application,
       :config_file_name => {
           :type     => :string,
           :default  => Puppet::Settings.default_config_file_name,
@@ -1177,14 +1097,9 @@ EOT
         :default    => nil,
         :desc       => "The address the agent should use to initiate requests.",
       },
-      :bindaddress => {
-        :default    => "*",
-        :desc       => "The address a listening server should bind to.",
-        :deprecated  => :completely,
-      }
   )
 
-  settings.define_settings(:environment,
+  define_settings(:environment,
     :manifest => {
       :default    => nil,
       :type       => :file_or_directory,
@@ -1227,7 +1142,7 @@ EOT
     }
   )
 
-  settings.define_settings(:master,
+  define_settings(:master,
     :user => {
       :default    => "puppet",
       :desc       => "The user Puppet Server will run as. Used to ensure
@@ -1277,19 +1192,6 @@ EOT
       by `puppet`, and should only be set if you're writing your own Puppet
       executable.",
     },
-    :masterhttplog => {
-      :default => "$logdir/masterhttp.log",
-      :type => :file,
-      :owner => "service",
-      :group => "service",
-      :mode => "0660",
-      :create => true,
-      :deprecated => :completely,
-      :desc => "Where the puppet master web server saves its access log. This is
-        only used when running a WEBrick puppet master. When puppet master is
-        running under a Rack server like Passenger, that web server will have
-        its own logging behavior."
-    },
     :masterport => {
       :default    => 8140,
       :desc       => "The default port puppet subcommands use to communicate
@@ -1297,23 +1199,13 @@ EOT
       overridden by more specific settings (see `ca_port`, `report_port`).",
     },
     :node_name => {
-      :default    => 'cert',
-      :type       => :enum,
-      :values     => ['cert', 'facter'],
-      :deprecated => :completely,
-      :hook       => proc { |val|
-        if val != 'cert'
-          Puppet.deprecation_warning("The node_name setting is deprecated and will be removed in a future release.")
-        end
-      },
+      :default    => "cert",
       :desc       => "How the puppet master determines the client's identity
       and sets the 'hostname', 'fqdn' and 'domain' facts for use in the manifest,
       in particular for determining which 'node' statement applies to the client.
       Possible values are 'cert' (use the subject's CN in the client's
       certificate) and 'facter' (use the hostname that the client
-      reported in its facts).
-
-      This setting is deprecated, please use explicit fact matching for classification.",
+      reported in its facts)",
     },
     :bucketdir => {
       :default => "$vardir/bucket",
@@ -1332,27 +1224,13 @@ EOT
       `puppet master`.  The `puppet master` command is deprecated and Puppet Server
       uses its own auth.conf that must be placed within its configuration directory.",
     },
-    :ca => {
-      :default    => true,
-      :type       => :boolean,
-      :desc       => "Whether the master should function as a certificate authority.",
-      :hook => proc do |value|
-        Puppet.deprecation_warning(_("The 'ca' setting is deprecated and will be removed in Puppet 6."))
-      end,
-    },
     :trusted_oid_mapping_file => {
       :default    => "$confdir/custom_trusted_oid_mapping.yaml",
       :type       => :file,
       :desc       => "File that provides mapping between custom SSL oids and user-friendly names"
     },
     :basemodulepath => {
-      :default => lambda do
-        if Puppet::Util::Platform.windows?
-          '$codedir/modules'
-        else
-          '$codedir/modules:/opt/puppetlabs/puppet/modules'
-        end
-      end,
+      :default => lambda { default_basemodulepath },
       :type => :path,
       :desc => "The search path for **global** modules. Should be specified as a
         list of directories separated by the system path separator character. (The
@@ -1363,6 +1241,14 @@ EOT
         any global directories. For more info, see
         <https://puppet.com/docs/puppet/latest/environments_about.html>",
     },
+    :vendormoduledir => {
+      :default => lambda { default_vendormoduledir },
+      :type => :string,
+      :desc => "The directory containing **vendored** modules. These modules will
+      be used by _all_ environments like those in the `basemodulepath`. The only
+      difference is that modules in the `basemodulepath` are pluginsynced, while
+      vendored modules are not",
+    },
     :ssl_client_header => {
       :default    => "HTTP_X_CLIENT_DN",
       :desc       => "The header containing an authenticated client's SSL DN.
@@ -1442,23 +1328,14 @@ EOT
       :desc       => "Where the fileserver configuration is stored.",
     },
     :strict_hostname_checking => {
-      :default    => true,
-      :type       => :boolean,
+      :default    => false,
       :desc       => "Whether to only search for the complete
-        hostname as it is in the certificate when searching for node information
-        in the catalogs or to match dot delimited segments of the cert's certname
-        and the hostname, fqdn, and/or domain facts.
-
-        This setting is deprecated and will be removed in a future release.",
-      :hook => proc { |val|
-        if val != true
-          Puppet.deprecation_warning("Setting strict_hostname_checking to false is deprecated and will be removed in a future release. Please use regular expressions in your node declarations or explicit fact matching for classification (though be warned that fact based classification may be considered insecure).")
-        end
-      }
+            hostname as it is in the certificate when searching for node information
+            in the catalogs.",
     }
   )
 
-  settings.define_settings(:device,
+  define_settings(:device,
     :devicedir =>  {
         :default  => "$vardir/devices",
         :type     => :directory,
@@ -1473,7 +1350,7 @@ EOT
     }
   )
 
-  settings.define_settings(:agent,
+  define_settings(:agent,
     :node_name_value => {
       :default => "$certname",
       :desc => "The explicit value used for the node name for all requests the agent
@@ -1574,12 +1451,30 @@ EOT
     :server => {
       :default => "puppet",
       :desc => "The puppet master server to which the puppet agent should connect.",
+      :call_hook => :on_initialize_and_write,
+      :hook => proc { |value|
+        if Puppet.settings.set_by_config?(:server) && Puppet.settings.set_by_config?(:server_list)
+          #TRANSLATOR 'server' and 'server_list' are setting names and should not be translated
+          message = _('Attempted to set both server and server_list.')
+          message += ' ' + _('Server setting will not be used.')
+          Puppet.deprecation_warning(message, :SERVER_DUPLICATION)
+        end
+      }
     },
     :server_list => {
       :default => [],
       :type => :server_list,
       :desc => "The list of puppet master servers to which the puppet agent should connect,
         in the order that they will be tried.",
+      :call_hook => :on_initialize_and_write,
+      :hook => proc { |value|
+        if Puppet.settings.set_by_config?(:server) && Puppet.settings.set_by_config?(:server_list)
+          #TRANSLATOR 'server' and 'server_list' are setting names and should not be translated
+          message = _('Attempted to set both server and server_list.')
+          message += ' ' + _('Server setting will not be used.')
+          Puppet.deprecation_warning(message, :SERVER_DUPLICATION)
+        end
+      }
     },
     :use_srv_records => {
       :default    => false,
@@ -1636,11 +1531,11 @@ EOT
           it with the `--no-client` option. #{AS_DURATION}",
     },
     :runtimeout => {
-      :default  => 0,
+      :default  => "1h",
       :type     => :duration,
       :desc     => "The maximum amount of time an agent run is allowed to take.
-          A Puppet agent run that exceeds this timeout will be aborted.
-          Defaults to 0, which is unlimited. #{AS_DURATION}",
+          A Puppet agent run that exceeds this timeout will be aborted. A value
+          of 0 disables the timeout. Defaults to 1 hour. #{AS_DURATION}",
     },
     :ca_server => {
       :default    => "$server",
@@ -1696,12 +1591,6 @@ EOT
         prior compilation. The list of missing objects is maintained per-environment and
         persists until the environment is cleared or the master is restarted.",
     },
-    :ignorecache => {
-      :default    => false,
-      :type       => :boolean,
-      :desc       => "This setting has no effect and will be removed in a future Puppet version.",
-      :deprecated => :completely,
-    },
     :splaylimit => {
       :default    => "$runinterval",
       :type       => :duration,
@@ -1739,19 +1628,6 @@ EOT
       :mode     => "0750",
       :desc     => "Where FileBucket files are stored locally."
     },
-    :configtimeout => {
-      :default  => "2m",
-      :type     => :duration,
-      :desc     => "How long the client should wait for the configuration to be retrieved
-        before considering it a failure. This setting is deprecated and has been replaced
-        by http_connect_timeout and http_read_timeout.
-        #{AS_DURATION}",
-      :deprecated => :completely,
-      :hook       => proc do |value|
-        Puppet[:http_connect_timeout] = value
-        Puppet[:http_read_timeout]    = value
-      end
-    },
     :report_server => {
       :default  => "$server",
       :desc     => "The server to send transaction reports to.",
@@ -1765,11 +1641,6 @@ EOT
       :type     => :boolean,
       :desc     => "Whether to send reports after every transaction.",
     },
-    :resubmit_facts => {
-      :default  => false,
-      :type     => :boolean,
-      :desc     => "Whether to send updated facts after every transaction.",
-    },
     :lastrunfile =>  {
       :default  => "$statedir/last_run_summary.yaml",
       :type     => :file,
@@ -1823,43 +1694,12 @@ EOT
       turn off waiting for certificates by specifying a time of 0, in which case
       puppet agent will exit if it cannot get a cert.
       #{AS_DURATION}",
-    },
-    :ordering => {
-      :type => :enum,
-      :values => ["manifest", "title-hash", "random"],
-      :default => "manifest",
-      :desc => "How unrelated resources should be ordered when applying a catalog.
-      Allowed values are `title-hash`, `manifest`, and `random`. This
-      setting affects puppet agent and puppet apply, but not puppet master.
-
-      * `manifest` (the default) will use the order in which the resources were
-        declared in their manifest files.
-      * `title-hash` (the default in 3.x) will order resources randomly, but
-        will use the same order across runs and across nodes. It is only of
-        value if you're migrating from 3.x and have errors running with
-        `manifest`.
-      * `random` will order resources randomly and change their order with each
-        run. This can work like a fuzzer for shaking out undeclared dependencies.
-
-      Regardless of this setting's value, Puppet will always obey explicit
-      dependencies set with the before/require/notify/subscribe metaparameters
-      and the `->`/`~>` chaining arrows; this setting only affects the relative
-      ordering of _unrelated_ resources.
-
-      This setting is deprecated, and will always have a value of `manifest` in
-      6.0 and up.",
-      :call_hook => :on_initialize_and_write,
-      :hook => proc { |value|
-        if value != "manifest"
-          Puppet.deprecation_warning(_('Setting %{name} is deprecated.') % { name: 'ordering' }, 'setting-ordering')
-        end
-      }
     }
   )
 
   # Plugin information.
 
-  settings.define_settings(
+  define_settings(
     :main,
     :plugindest => {
       :type       => :directory,
@@ -1894,17 +1734,6 @@ EOT
       is used for retrieval, so anything that is a valid file source can
       be used here.",
     },
-
-    :pluginsync => {
-      :default    => true,
-      :type       => :boolean,
-      :desc       => "Whether plugins should be synced with the central server. This setting is
-        deprecated.",
-      :hook => proc { |value|
-        #TRANSLATORS 'pluginsync' is a setting and should not be translated
-        Puppet.deprecation_warning(_("Setting 'pluginsync' is deprecated."))
-      }
-    },
     :pluginsignore => {
         :default  => ".svn CVS .git .hg",
         :desc     => "What files to ignore when pulling down plugins.",
@@ -1913,7 +1742,7 @@ EOT
 
   # Central fact information.
 
-    settings.define_settings(
+    define_settings(
     :main,
     :factpath => {
       :type     => :path,
@@ -1930,7 +1759,7 @@ EOT
     }
   )
 
-  settings.define_settings(
+  define_settings(
     :transaction,
     :tags => {
       :default    => "",
@@ -1958,7 +1787,7 @@ EOT
     }
   )
 
-    settings.define_settings(
+    define_settings(
     :main,
     :external_nodes => {
         :default  => "none",
@@ -1983,7 +1812,7 @@ EOT
     }
     )
 
-        settings.define_settings(
+        define_settings(
         :ldap,
     :ldapssl => {
       :default  => false,
@@ -2001,11 +1830,11 @@ EOT
     },
     :ldapserver => {
       :default  => "ldap",
-      :desc     => "The LDAP server.  Only used if `node_terminus` is set to `ldap`.",
+      :desc     => "The LDAP server.",
     },
     :ldapport => {
       :default  => 389,
-      :desc     => "The LDAP port.  Only used if `node_terminus` is set to `ldap`.",
+      :desc     => "The LDAP port.",
     },
 
     :ldapstring => {
@@ -2052,7 +1881,7 @@ EOT
     }
   )
 
-  settings.define_settings(:master,
+  define_settings(:master,
     :storeconfigs => {
       :default  => false,
       :type     => :boolean,
@@ -2085,7 +1914,7 @@ EOT
     }
   )
 
-  settings.define_settings(:parser,
+  define_settings(:parser,
    :max_errors => {
      :default => 10,
      :desc => <<-'EOT'
@@ -2127,7 +1956,7 @@ EOT
     EOT
     }
   )
-  settings.define_settings(:puppetdoc,
+  define_settings(:puppetdoc,
     :document_all => {
         :default  => false,
         :type     => :boolean,
@@ -2136,10 +1965,10 @@ EOT
     }
   )
 
-  settings.define_settings(
+  define_settings(
     :main,
     :rich_data => {
-      :default  => false,
+      :default  => true,
       :type     => :boolean,
       :hook    => proc do |value|
         envs = Puppet.lookup(:environments) { nil }
@@ -2147,11 +1976,11 @@ EOT
       end,
       :desc     => <<-'EOT'
         Enables having extended data in the catalog by storing them as a hash with the special key
-        `__pcore_type__`. When enabled, resource containing values of the data types `Binary`, `Regexp`,
+        `__ptype`. When enabled, resource containing values of the data types `Binary`, `Regexp`,
         `SemVer`, `SemVerRange`, `Timespan` and `Timestamp`, as well as instances of types derived
         from `Object` retain their data type.
       EOT
     }
   )
-  end
+
 end
