@@ -11,14 +11,6 @@ class Puppet::Provider::Package::Windows
 
     attr_reader :name, :version
 
-    REG_DISPLAY_VALUE_NAMES = [ 'DisplayName', 'QuietDisplayName' ]
-
-    def self.reg_value_names_to_load
-      REG_DISPLAY_VALUE_NAMES |
-      MsiPackage::REG_VALUE_NAMES |
-      ExePackage::REG_VALUE_NAMES
-    end
-
     # Enumerate each package. The appropriate package subclass
     # will be yielded.
     def self.each(&block)
@@ -45,7 +37,7 @@ class Puppet::Provider::Package::Windows
             open(hive, 'Software\Microsoft\Windows\CurrentVersion\Uninstall', mode) do |uninstall|
               each_key(uninstall) do |name, wtime|
                 open(hive, "#{uninstall.keyname}\\#{name}", mode) do |key|
-                  yield key, values_by_name(key, reg_value_names_to_load)
+                  yield key, values(key)
                 end
               end
             end
@@ -79,7 +71,7 @@ class Puppet::Provider::Package::Windows
 
     def self.replace_forward_slashes(value)
       if value.include?('/')
-        value.gsub!('/', "\\")
+        value = value.gsub('/', "\\")
         Puppet.debug('Package source parameter contained /s - replaced with \\s')
       end
       value
